@@ -244,6 +244,7 @@ struct raft_request_vote
     raft_index last_log_index; /* Index of candidate's last log entry. */
     raft_index last_log_term;  /* Term of log entry at last_log_index. */
     bool disrupt_leader;       /* True if current leader should be discarded. */
+    bool pre_vote;             /* True if this is a pre-vote request. */
 };
 
 /**
@@ -640,6 +641,8 @@ struct raft
         {
             unsigned randomized_election_timeout; /* Timer expiration. */
             bool *votes;                          /* Vote results. */
+            bool disrupt_leader;                  /* For leadership transfer */
+            bool in_pre_vote;                     /* True in pre-vote phase. */
         } candidate_state;
         struct
         {
@@ -687,6 +690,10 @@ struct raft
      * error occurred.
      */
     char errmsg[RAFT_ERRMSG_BUF_SIZE];
+
+    /* Whether to use pre-vote to avoid disconnected servers disrupting the
+     * current leader, as described in 4.2.3 and 9.6. */
+    bool pre_vote;
 };
 
 RAFT_API int raft_init(struct raft *r,
@@ -761,6 +768,11 @@ RAFT_API void raft_set_heartbeat_timeout(struct raft *r, unsigned msecs);
  */
 RAFT_API void raft_set_snapshot_threshold(struct raft *r, unsigned n);
 
+/**
+ * Enable or disable pre-vote support. Pre-vote is turned off by default.
+ */
+RAFT_API void raft_set_pre_vote(struct raft *r, bool enabled);
+
 /**
  * Number of outstanding log entries to keep in the log after a snapshot has
  * been taken. This avoids sending snapshots when a follower is behind by just a
